#include once "mud-gui/widgets/LineEdit.bi"

namespace ui

constructor LineEdit( byval parent as BaseWidget ptr = 0, byval _text_ as string = "" )

	if parent <> 0 then
	
		if parent->cl <> layout_t.canHaveLayout then ShowError("LineEdit widget must have a valid parent.")
		SetParent(parent)

	end if
	
	setText = _text_

	(BaseObject).id = ui.widgetids.lineedit
	(BaseObject).cl = layout_t.noLayout
	
	connectSignal(@this,repaint,LineEdit.PaintEvent)
	connectSignal(@this,destroy,LineEdit.Destroy)
	connectSignal(@this,resize,LineEdit.resizeEvent)
	connectSignal(@this,timerExpire,LineEdit.timerEvent)
	connectSignal(@this,keystroke,LineEdit.keystrokeEvent)
	connectSignal(@this,clicked,LineEdit.clickEvent)
	
	addTimer( toBaseWidget(@this), 0.0, timer_e.repeat )
	setEditable = 1

end constructor

IMP_BYREF_PROPERTY( LineEdit, text, string )
IMP_PROPERTY( LineEdit, editable, ubyte )

property LineEdit.minimumWidth( ) as integer
	return (BaseObject).minwidth
end property

property LineEdit.setMinimumWidth( byval _t_ as integer )
	(BaseObject).minwidth = _t_
end property

property LineEdit.minimumHeight( ) as integer
	return (BaseObject).minheight
end property

property LineEdit.setMinimumHeight( byval _t_ as integer )
	(BaseObject).minheight = _t_
end property

STATIC_SLOT(LineEdit.clickEvent)

	if cast( LineEdit ptr, ref )->editable = 1 then
	if setFocus(0) <> ref then setFocus(ref)
	end if
	return 1

ENDSLOT

STATIC_SLOT(LineEdit.Destroy)

	var le = cast(LineEdit ptr, ref)
	
	le->setText = ""
	
	return 0
	
ENDSLOT

STATIC_SLOT(LineEdit.PaintEvent)

	'ref->resize(ref,e)
	var le = cast(LineEdit ptr, ref)
	put ( ref->x, ref->y), ref->buffer, PSET

	if le->m_cursor = 1 then
		if le->editable = 1 then
		draw string (ref->x+3,ref->y+3), le->text & "[", rgb(0,0,0)
		else
			le->m_cursor = 0
		end if
	else
		draw string (ref->x+3,ref->y+3), le->text, rgb(0,0,0)
	end if
	
	return 0
	
ENDSLOT

STATIC_SLOT(LineEdit.resizeEvent)


	if ref->buffer = 0 then ref->buffer = imagecreate( ref->width, ref->height, rgb(&hc9,&hc9,&hc9) )

	if cast(LineEdit ptr, ref)->editable = 1 then
		line ref->buffer,(1,1)-(ref->width-2,ref->height-2), rgb(255,255,255), BF
		line ref->buffer,(1,1)-(ref->width-2,ref->height-2), rgb(0,0,0), B
	else
		line ref->buffer,(1,1)-(ref->width-2,ref->height-2), rgb(220,220,220), BF
		line ref->buffer,(1,1)-(ref->width-2,ref->height-2), rgb(0,0,0), B
	end if
	
	return 0

ENDSLOT

STATIC_SLOT(LineEdit.timerEvent)

	var le = cast(LineEdit ptr, ref)
	if le->editable = 1 then
		le->m_cursor = iif(le->m_cursor=1,0,1)
	else
		le->m_cursor = 0
	end if
	return 0

ENDSLOT

STATIC_SLOT(LineEdit.keystrokeEvent)

	var le = cast(LineEdit ptr, ref)
	if le->editable = 1 then
	select case e->ascii
	case 32 to 126
		le->setText = le->text & chr(e->ascii)
	case else
		select case e->scancode
		case fb.SC_ESCAPE
			'todo: send cancel signal
		case fb.SC_ENTER
			'todo: send activate signal
		case fb.SC_BACKSPACE
			le->setText = left(le->text, len(le->text)-1)
		'todo: arrow keys and other editing functions
		end select
	end select
	end if

	return 0
	
ENDSLOT

end namespace
